<?php
# Set namespace
namespace DataAccess;

# Set Laravel class alias
use Illuminate\Database\Capsule\Manager as Capsule;
use opensrs\Exception;

/**
 * Database Access Class
 */
class DataAccessOln {
    /**
     * Get SSL Product Data
     * @param int $service_id::SSL product ID
     * @return mixed $response::SSL product data or return false
     */
    public static function GetSSLProduct($id){
        try{
            $response = Capsule::table('tblsslorders')
                ->where('id', $id)
                ->first();
            return $response;
        } catch (Exception $e) {
            return False;
        }
    }

    /**
     * Get Service ID
     * @param int $service_id::Service ID
     * @return mixed::Service ID info or error status
     */
    public static function GetServiceID($service_id) {
        try {
            $results = Capsule::table('tblhosting')
                ->where('id', $service_id)
                ->first();
            return $results;
        } catch (Exception $e) {
            return False;
        }
    }

    /**
     * Create SSL Order
     * @param array $params::Data from WHMCS
     * @return int $id::Database order ID
     */
    public static function CreateOrder($params) {
        # Try to get existing order
        try {
            $result = Capsule::table('tblsslorders')
                ->where('id', $params['orderid'])
                ->first();
        } catch (Exception $e) {
            return False;
        }

        # GC::Check response
        if(empty($result)) {
            # Insert new SSL order
            try {
                $id = Capsule::table('tblsslorders')
                    ->insertGetId([
                        'userid' => $params['clientsdetails']['userid'],
                        'serviceid' => $params['serviceid'],
                        'module' => 'onlinenic_ssl',
                        'certtype' => $params['configoption3'],
                        'status' => 'Awaiting Configuration'
                    ]);
                return $id;
            } catch (Exception $e) {
                return False;
            }
        } else {
            # Update existing SSL order
            try {
                Capsule::table('tblsslorders')
                    ->where('id', $params['orderid'])
                    ->update([
                        'userid' => $params['clientsdetails']['userid'],
                        'serviceid' => $params['serviceid'],
                        'module' => 'onlinenic_ssl',
                        'certtype' => $params['configoption3'],
                        'status' => 'Awaiting Configuration'
                    ]);
                return $result->id;
            } catch (Exception $e) {
                return False;
            }
        }
    }

    /**
     * Update SSL Order
     * @param int $service_id::Service ID
     * @param string $column::Column name
     * @param mixed $value::Value to insert
     * @return boolean::Operation status
     */
    public static function UpdateOrder($service_id, $column, $value) {
        try {
            Capsule::table('tblsslorders')
                ->where('id', $service_id)
                ->update([$column => $value]);
            return True;
        } catch (Exception $e) {
            return False;
        }
    }

    /**
     * Update SSL Order Config Data
     */
    public static function UpdateConfigData($service_id, $config_data) {
        try {
            Capsule::table('tblsslorders')
                ->where('id', $service_id)
                ->update(['configdata' => json_encode($config_data)]);
            return True;
        } catch (Exception $e) {
            return False;
        }
    }

    /**
     * Update Product Domain Name
     * @param int $service_id::Service ID
     * @param string $domain_name::Product's domain name
     * @return boolean::Operations status
     */
    public static function UpdateDomain($service_id, $domain_name) {
        try {
            $response = Capsule::table('tblhosting')
                ->where('id', $service_id)
                ->update(['domain' => $domain_name]);
            return True;
        } catch (Exception $e) {
            return False;
        }
    }

    /**
     * Update Product Domain Name
     * @param int $service_id::Service ID
     * @param string $status::Product's status
     * @return boolean::Operations status
     */
    public static function UpdateStatus($service_id, $status) {
        try {
            Capsule::table('tblsslorders')
                ->where('serviceid', $service_id)
                ->update(['status' => $status]);
            return True;
        } catch (Exception $e) {
            return False;
        }
    }

    /**
     * Update Product Billing Cycles
     * @param int $service_id::Service ID
     * @return boolean::Operation Status
     */
    public static function UpdateBilling($service_id, $start_date, $expiry_date) {
        try {
            Capsule::table('tblhosting')
                ->where('id', $service_id)
                ->update(['regdate' => $start_date, 'nextduedate' => $expiry_date, 'nextinvoicedate' => $expiry_date]);
            return True;
        } catch (Exception $e) {
            return False;
        }
    }

    /**
     * Create Custom SSL Config Email Template
     * @return boolean::Operation Status
     */
    public static function CreateEmailTemplate() {
        # Create new email template if it does not exist
        try {
            $email_template = Capsule::table('tblemailtemplates')
                ->where('name', 'OpenSRS - SSL Certificate Configuration Required')
                ->first();
        } catch (Exception $e) {
            return false;
        }

        # GC::Check if email template exists
        if(empty($email_template)) {
            $message = '
                <p> Dear {$client_name}, </p>
                <p> Thank you for your order for an SSL Certificate. Before you can use your certificate, it requires 
                configuration which can be done at the URL below. </p>
                <p> {$ssl_configuration_link} </p>
                <p> Instructions are provided throughout the process but if you experience any problems or have any 
                questions, please open a ticket for assistance. </p>
                <p> {$signature} </p>
                ';

            # Insert new email template
            try {
                Capsule::table('tblemailtemplates')
                    ->insert([
                        [
                            'type' => 'product',
                            'name' => 'OpenSRS - SSL Certificate Configuration Required',
                            'subject' => 'SSL Certificate Configuration Required',
                            'message' => $message,
                            'fromname' => '',
                            'fromemail' => '',
                            'disabled' => '',
                            'custom' => '1',
                            'language' => '',
                            'copyto' => '',
                            'plaintext' => '0'
                        ],
                    ]);
                return True;
            } catch (Exception $e) {
                return False;
            }
        }

    }

    /**
     * Reset Product User info
     * @param string $username::Username
     * @param string $password::Password
     * @param int $service_id::Service Id
     * @return boolean::Operation status
     */
    public static function ResetUserInfo($username, $password, $service_id) {
        # Reset username/password for table 'tblhosting'
        try {
            Capsule::table('tblhosting')
                ->where('id', $service_id)
                ->update([
                    'username' => $username,
                    'password' => $password
                ]);
            return True;
        } catch (Exception $e) {
            Return False;
        }
    }

    /**
     * Get User Details
     * @param Int $user_id::User ID
     * @return array $result::User details
     */
    public static function GetUser($user_id) {
        try {
            $results = Capsule::table('tblclients')
                ->where('id', $user_id)
                ->first();
            return $results;
        } catch (Exception $e) {
            return False;
        }
    }

    /**
     * Get System Language Setting
     * @return string $lang::System default language setting
     */
    public static function GetLanguage() {
        try {
            $results = Capsule::table('tblconfiguration')
                ->select('Language');
            return $results->Language;
        } catch (Exception $e) {
            return False;
        }
    }

    /**
     * Terminate SSL Order
     * @param int $service_id::Service ID
     * @return boolean::Operation status
     */
    public static function TerminateSSLOrder($service_id) {
        try {
            Capsule::table('tblhosting')
                ->where('id', $service_id)
                ->update(['domainstatus' => 'Terminated']);
            return True;
        } catch (Exception $e) {
            return False;
        }
    }

    /**
     * Get All SSL Orders
     * @return mixed::All OpenSRS SSL orders or operation status
     */
    public static function GetAllOrders() {
        try {
            return Capsule::table('tblsslorders')
                ->where('module', 'opensrs_ssl')
                ->get();
        } catch (Exception $e) {
            return False;
        }
    }

    /**
     * Update Next Due Date
     * @param string $date::Next due date
     * @return boolean::Operation status
     */
    public static function UpdateNextDueDate($service_id, $date) {
        try {
            Capsule::table('tblhosting')
                ->where('id', $service_id)
                ->update(['nextduedate' => $date]);
            return True;
        } catch (Exception $e) {
            return False;
        }
    }
}
